/* ==========================================================================
 * Copyright 2005 Mevenide Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * =========================================================================
 */

package org.codehaus.mevenide.continuum.options;

import java.awt.Dimension;
import java.net.MalformedURLException;
import java.net.URL;
import org.apache.commons.lang.StringUtils;
import org.codehaus.mevenide.continuum.ServerInfo;
import org.openide.util.Exceptions;

/**
 *
 * @author Milos Kleint (mkleint@codehaus.org)
 */
public class SingleServer extends javax.swing.JPanel {
    
    /** Creates new form SingleServer */
    public SingleServer() {
        initComponents();
        setPreferredSize(new Dimension(350, 200));
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        xmlRpcUrlLabel = new javax.swing.JLabel();
        xmlRpcUrlTextField = new javax.swing.JTextField();
        userLabel = new javax.swing.JLabel();
        userTextField = new javax.swing.JTextField();
        passwordLabel = new javax.swing.JLabel();
        webUrlTextField = new javax.swing.JTextField();
        webUrlLabel = new javax.swing.JLabel();
        passwordField = new javax.swing.JPasswordField();

        xmlRpcUrlLabel.setLabelFor(xmlRpcUrlTextField);
        xmlRpcUrlLabel.setText("XmlRpc-Url :");

        xmlRpcUrlTextField.setText("http://localhost:9090/xmlrpc");
        xmlRpcUrlTextField.setToolTipText("The continuum xml rpc URL. Usually http://localhost:8080/continuum/xmlrpc");

        userLabel.setLabelFor(userTextField);
        userLabel.setText("User :");

        userTextField.setToolTipText("Leave empty for guest access");
        userTextField.setMinimumSize(new java.awt.Dimension(30, 19));
        userTextField.setPreferredSize(new java.awt.Dimension(40, 19));

        passwordLabel.setLabelFor(passwordField);
        passwordLabel.setText("Password :");

        webUrlTextField.setText("http://localhost:9090");
        webUrlTextField.setToolTipText("The continuum admin URL. Usually http://localhost:8080/continuum ");

        webUrlLabel.setLabelFor(webUrlTextField);
        webUrlLabel.setText("Web-Url :");

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(xmlRpcUrlLabel)
                    .add(webUrlLabel)
                    .add(userLabel)
                    .add(passwordLabel))
                .add(37, 37, 37)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(userTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 275, Short.MAX_VALUE)
                    .add(webUrlTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 275, Short.MAX_VALUE)
                    .add(xmlRpcUrlTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 275, Short.MAX_VALUE)
                    .add(passwordField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 275, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(xmlRpcUrlLabel)
                    .add(xmlRpcUrlTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(webUrlLabel)
                    .add(webUrlTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(userLabel)
                    .add(userTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(passwordLabel)
                    .add(passwordField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPasswordField passwordField;
    private javax.swing.JLabel passwordLabel;
    private javax.swing.JLabel userLabel;
    private javax.swing.JTextField userTextField;
    private javax.swing.JLabel webUrlLabel;
    private javax.swing.JTextField webUrlTextField;
    private javax.swing.JLabel xmlRpcUrlLabel;
    private javax.swing.JTextField xmlRpcUrlTextField;
    // End of variables declaration//GEN-END:variables
    
    public ServerInfo getServerInfo() throws MalformedURLException {
        ServerInfo serverInfo = new ServerInfo();
        serverInfo.setXmlRpcUrl(new URL(xmlRpcUrlTextField.getText().trim()));
        serverInfo.setWebUrl(new URL(webUrlTextField.getText().trim()));
        String login = userTextField.getText().trim();
        if (StringUtils.isEmpty(login)) {
            serverInfo.setUser(null);
            serverInfo.setPassword(null);
        } else {
            serverInfo.setUser(login);
            serverInfo.setPassword(new String(passwordField.getPassword()));
        }

        return serverInfo;
    }
        }
